/*---------------------------------------------------------------------------*/

/*!  \file  correlation.h
     \brief  calculate the correlation function for Gaussian chains.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_CORRELATION_H
#define INCLUDE_CORRELATION_H

#include "polymer.h"

/*---------------------------------------------------------------------------*/
/*!  \brief  self correlation function of the \a i subchain.
     \param  i  index of a subchain
     \param  q2  square of wave number (q^2)  */
double h1(polymer_chain *chain,int i,double q2);
/*!  \brief  two body correlation function between the \a i and \a j subchain.
     \param  i  index of a subchain
     \param  j  index of a subchain
     \param  q2  square of wave number (q^2)  */
double h2(polymer_chain *chain,int i,int j,double q2);
/*!  \brief  calculate the coefficient matrix of q^2 at q^2 = 0.
     \param  chain  polymer chain
     \param  H01  coefficient matrix of q^2 at q^2 = 0  */
void calculate_H01(polymer_chain *chain,double **H01);
/*!  \brief  calculate the coefficient matrix of q^4 at q^2 = 0.
     \param  chain  polymer chain
     \param  H02  coefficient matrix of q^4 at q^2 = 0  */
void calculate_H02(polymer_chain *chain,double **H02);
/*!  \brief  calculate the coefficient matrix of 1 / q^2 at q^2 = infinity.
     \param  chain  polymer chain
     \param  Hi2  coefficient matrix of 1 / q^2 at q^2 = infinity  */
void calculate_Hi2(polymer_chain *chain,double **Hi2);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
