/*---------------------------------------------------------------------------*/

/*!  \file  iofiles.c
     \brief  input / output files.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005,2006 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include "xmalloc.h"
#include "iofiles.h"

/*---------------------------------------------------------------------------*/
simulation_iofiles *allocate_simulation_iofiles(void)
{
    simulation_iofiles *iofiles;

    iofiles = xmalloc(sizeof(simulation_iofiles));

    iofiles->wisdom = NULL;

    iofiles->psi_output = NULL;
    iofiles->density_output = NULL;
    iofiles->chemical_potential_output = NULL;
    iofiles->free_energy_output = NULL;
    iofiles->geometry_output = NULL;
    iofiles->dx_output = NULL;

    iofiles->psi_template = NULL;
    iofiles->density_template = NULL;
    iofiles->chemical_potential_template = NULL;
    iofiles->free_energy_template = NULL;
    iofiles->geometry_template = NULL;
    iofiles->dx_template = NULL;

    return iofiles;
}

/*---------------------------------------------------------------------------*/
void free_simulation_iofiles(simulation_iofiles *iofiles)
{
    free(iofiles->wisdom);

    free(iofiles->psi_output);
    free(iofiles->density_output);
    free(iofiles->chemical_potential_output);
    free(iofiles->free_energy_output);
    free(iofiles->geometry_output);
    free(iofiles->dx_output);

    free(iofiles->psi_template);
    free(iofiles->density_template);
    free(iofiles->chemical_potential_template);
    free(iofiles->free_energy_template);
    free(iofiles->geometry_template);
    free(iofiles->dx_template);

    free(iofiles);
}

/*---------------------------------------------------------------------------*/
