/*---------------------------------------------------------------------------*/

/*!  \file  iofile.h
     \brief  input / output files.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_IOFILES_H
#define INCLUDE_IOFILES_H

/*---------------------------------------------------------------------------*/
/*!  \brief  input/output files for simulator.  */
typedef struct simulation_iofiles
{
        /*!  \brief  input / output wisdom for FFTW.  */
    char *wisdom;

        /*!  \brief  output files.  */
    char *psi_output;
    char *density_output;
    char *chemical_potential_output;
    char *free_energy_output;
    char *geometry_output;
    char *dx_output;

        /*!  \brief  output file name templates for sequential save.  */
    char *psi_template;
    char *density_template;
    char *chemical_potential_template;
    char *free_energy_template;
    char *geometry_template;
    char *dx_template;
} simulation_iofiles;

/*---------------------------------------------------------------------------*/
/*!  \brief  allocate a new simulation condition.  */
simulation_iofiles *allocate_simulation_iofiles(void);
/*!  \brief  free a simulation condition.
     \param  iofile  simulation input / output files to free  */
void free_simulation_iofiles(simulation_iofiles *iofiles);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
