/*---------------------------------------------------------------------------*/

/*!  \file  laplacian.c
     \brief  Laplacian operator for fields.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/

/*  DO NOT EDIT THIS FILE. This file is generated from laplacian.c.m4.  */

#include "common.h"

#include "laplacian.h"

/*---------------------------------------------------------------------------*/
void operate_L_1d(field_geometry *geometry,double *psi,double *L_psi)
{
    int i;
    int nx;
    double lx,dx,dx2;
    int ip,im;

    nx = geometry->nx;
    lx = geometry->lx;
    dx = lx / nx;
    dx2 = dx * dx;

    for(i = 0;i < 1;i++)
    {
        im = nx - 1;
        ip = 1 % nx;
        L_psi[i] = (psi[im] + psi[ip] - 2 * psi[i]) / dx2;
    }
    for(;i < (nx - 1);i++)
    {
        im = i - 1;
        ip = i + 1;
        L_psi[i] = (psi[im] + psi[ip] - 2 * psi[i]) / dx2;
    }
    for(;i < nx;i++)
    {
        im = nx - 2;
        ip = 0;
        L_psi[i] = (psi[im] + psi[ip] - 2 * psi[i]) / dx2;
    }
}

/*---------------------------------------------------------------------------*/
void operate_L_2d(field_geometry *geometry,double *psi,double *L_psi)
{
    int i,j,nx,ny;
    int ip,im,jp,jm;
    double lx,ly,dx,dy,dx2,dy2;

    nx = geometry->nx;
    ny = geometry->ny;
    lx = geometry->lx;
    ly = geometry->ly;
    dx = lx / nx;
    dy = ly / ny;
    dx2 = dx * dx;
    dy2 = dy * dy;

    for(i = 0;i < 1;i++)
    {
        im = nx - 1;
        ip = 1 % nx;
        for(j = 0;j < 1;j++)
        {
            jm = ny - 1;
            jp = 1 % ny;
            L_psi[i * ny + j] =
                  (psi[ip * ny + j] + psi[im * ny + j]
                   - 2 * psi[i * ny + j]) / dx2
                + (psi[i * ny + jp] + psi[i * ny + jm]
                   - 2 * psi[i * ny + j]) / dy2;
        }
        for(;j < (ny - 1);j++)
        {
            jm = j - 1;
            jp = j + 1;
            L_psi[i * ny + j] =
                  (psi[ip * ny + j] + psi[im * ny + j]
                   - 2 * psi[i * ny + j]) / dx2
                + (psi[i * ny + jp] + psi[i * ny + jm]
                   - 2 * psi[i * ny + j]) / dy2;
        }
        for(;j < ny;j++)
        {
            jm = ny - 2;
            jp = 0;
            L_psi[i * ny + j] =
                  (psi[ip * ny + j] + psi[im * ny + j]
                   - 2 * psi[i * ny + j]) / dx2
                + (psi[i * ny + jp] + psi[i * ny + jm]
                   - 2 * psi[i * ny + j]) / dy2;
        }
    }
    for(;i < (nx - 1);i++)
    {
        im = i - 1;
        ip = i + 1;
        for(j = 0;j < 1;j++)
        {
            jm = ny - 1;
            jp = 1 % ny;
            L_psi[i * ny + j] =
                  (psi[ip * ny + j] + psi[im * ny + j]
                   - 2 * psi[i * ny + j]) / dx2
                + (psi[i * ny + jp] + psi[i * ny + jm]
                   - 2 * psi[i * ny + j]) / dy2;
        }
        for(;j < (ny - 1);j++)
        {
            jm = j - 1;
            jp = j + 1;
            L_psi[i * ny + j] =
                  (psi[ip * ny + j] + psi[im * ny + j]
                   - 2 * psi[i * ny + j]) / dx2
                + (psi[i * ny + jp] + psi[i * ny + jm]
                   - 2 * psi[i * ny + j]) / dy2;
        }
        for(;j < ny;j++)
        {
            jm = ny - 2;
            jp = 0;
            L_psi[i * ny + j] =
                  (psi[ip * ny + j] + psi[im * ny + j]
                   - 2 * psi[i * ny + j]) / dx2
                + (psi[i * ny + jp] + psi[i * ny + jm]
                   - 2 * psi[i * ny + j]) / dy2;
        }
    }
    for(;i < nx;i++)
    {
        im = nx - 2;
        ip = 0;
        for(j = 0;j < 1;j++)
        {
            jm = ny - 1;
            jp = 1 % ny;
            L_psi[i * ny + j] =
                  (psi[ip * ny + j] + psi[im * ny + j]
                   - 2 * psi[i * ny + j]) / dx2
                + (psi[i * ny + jp] + psi[i * ny + jm]
                   - 2 * psi[i * ny + j]) / dy2;
        }
        for(;j < (ny - 1);j++)
        {
            jm = j - 1;
            jp = j + 1;
            L_psi[i * ny + j] =
                  (psi[ip * ny + j] + psi[im * ny + j]
                   - 2 * psi[i * ny + j]) / dx2
                + (psi[i * ny + jp] + psi[i * ny + jm]
                   - 2 * psi[i * ny + j]) / dy2;
        }
        for(;j < ny;j++)
        {
            jm = ny - 2;
            jp = 0;
            L_psi[i * ny + j] =
                  (psi[ip * ny + j] + psi[im * ny + j]
                   - 2 * psi[i * ny + j]) / dx2
                + (psi[i * ny + jp] + psi[i * ny + jm]
                   - 2 * psi[i * ny + j]) / dy2;
        }
    }
}

/*---------------------------------------------------------------------------*/
void operate_L_3d(field_geometry *geometry,double *psi,double *L_psi)
{
    int i,j,k;
    int nx,ny,nz;
    int ip,im,jp,jm,kp,km;
    double lx,ly,lz,dx,dy,dz,dx2,dy2,dz2;

    nx = geometry->nx;
    ny = geometry->ny;
    nz = geometry->nz;
    lx = geometry->lx;
    ly = geometry->ly;
    lz = geometry->lz;
    dx = lx / nx;
    dy = ly / ny;
    dz = lz / nz;
    dx2 = dx * dx;
    dy2 = dy * dy;
    dz2 = dz * dz;

    for(i = 0;i < 1;i++)
    {
        im = nx - 1;
        ip = 1 % nx;
        for(j = 0;j < 1;j++)
        {
            jm = ny - 1;
            jp = 1 % ny;
            for(k = 0;k < 1;k++)
            {
                km = nz - 1;
                kp = 1 % nz;
                L_psi[(i * ny + j) * nz + k] =
                      (psi[(ip * ny + j) * nz + k]
                       + psi[(im * ny + j) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dx2
                    + (psi[(i * ny + jp) * nz + k]
                       + psi[(i * ny + jm) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dy2
                    + (psi[(i * ny + j) * nz + kp]
                       + psi[(i * ny + j) * nz + km]
                       - 2 * psi[(i * ny + j) * nz + k]) / dz2;
            }
            for(;k < (nz - 1);k++)
            {
                km = k - 1;
                kp = k + 1;
                L_psi[(i * ny + j) * nz + k] =
                      (psi[(ip * ny + j) * nz + k]
                       + psi[(im * ny + j) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dx2
                    + (psi[(i * ny + jp) * nz + k]
                       + psi[(i * ny + jm) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dy2
                    + (psi[(i * ny + j) * nz + kp]
                       + psi[(i * ny + j) * nz + km]
                       - 2 * psi[(i * ny + j) * nz + k]) / dz2;
            }
            for(;k < nz;k++)
            {
                km = nz - 2;
                kp = 0;
                L_psi[(i * ny + j) * nz + k] =
                      (psi[(ip * ny + j) * nz + k]
                       + psi[(im * ny + j) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dx2
                    + (psi[(i * ny + jp) * nz + k]
                       + psi[(i * ny + jm) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dy2
                    + (psi[(i * ny + j) * nz + kp]
                       + psi[(i * ny + j) * nz + km]
                       - 2 * psi[(i * ny + j) * nz + k]) / dz2;
            }
        }
        for(;j < (ny - 1);j++)
        {
            jm = j - 1;
            jp = j + 1;
            for(k = 0;k < 1;k++)
            {
                km = nz - 1;
                kp = 1 % nz;
                L_psi[(i * ny + j) * nz + k] =
                      (psi[(ip * ny + j) * nz + k]
                       + psi[(im * ny + j) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dx2
                    + (psi[(i * ny + jp) * nz + k]
                       + psi[(i * ny + jm) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dy2
                    + (psi[(i * ny + j) * nz + kp]
                       + psi[(i * ny + j) * nz + km]
                       - 2 * psi[(i * ny + j) * nz + k]) / dz2;
            }
            for(;k < (nz - 1);k++)
            {
                km = k - 1;
                kp = k + 1;
                L_psi[(i * ny + j) * nz + k] =
                      (psi[(ip * ny + j) * nz + k]
                       + psi[(im * ny + j) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dx2
                    + (psi[(i * ny + jp) * nz + k]
                       + psi[(i * ny + jm) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dy2
                    + (psi[(i * ny + j) * nz + kp]
                       + psi[(i * ny + j) * nz + km]
                       - 2 * psi[(i * ny + j) * nz + k]) / dz2;
            }
            for(;k < nz;k++)
            {
                km = nz - 2;
                kp = 0;
                L_psi[(i * ny + j) * nz + k] =
                      (psi[(ip * ny + j) * nz + k]
                       + psi[(im * ny + j) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dx2
                    + (psi[(i * ny + jp) * nz + k]
                       + psi[(i * ny + jm) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dy2
                    + (psi[(i * ny + j) * nz + kp]
                       + psi[(i * ny + j) * nz + km]
                       - 2 * psi[(i * ny + j) * nz + k]) / dz2;
            }
        }
        for(;j < ny;j++)
        {
            jm = ny - 2;
            jp = 0;
            for(k = 0;k < 1;k++)
            {
                km = nz - 1;
                kp = 1 % nz;
                L_psi[(i * ny + j) * nz + k] =
                      (psi[(ip * ny + j) * nz + k]
                       + psi[(im * ny + j) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dx2
                    + (psi[(i * ny + jp) * nz + k]
                       + psi[(i * ny + jm) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dy2
                    + (psi[(i * ny + j) * nz + kp]
                       + psi[(i * ny + j) * nz + km]
                       - 2 * psi[(i * ny + j) * nz + k]) / dz2;
            }
            for(;k < (nz - 1);k++)
            {
                km = k - 1;
                kp = k + 1;
                L_psi[(i * ny + j) * nz + k] =
                      (psi[(ip * ny + j) * nz + k]
                       + psi[(im * ny + j) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dx2
                    + (psi[(i * ny + jp) * nz + k]
                       + psi[(i * ny + jm) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dy2
                    + (psi[(i * ny + j) * nz + kp]
                       + psi[(i * ny + j) * nz + km]
                       - 2 * psi[(i * ny + j) * nz + k]) / dz2;
            }
            for(;k < nz;k++)
            {
                km = nz - 2;
                kp = 0;
                L_psi[(i * ny + j) * nz + k] =
                      (psi[(ip * ny + j) * nz + k]
                       + psi[(im * ny + j) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dx2
                    + (psi[(i * ny + jp) * nz + k]
                       + psi[(i * ny + jm) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dy2
                    + (psi[(i * ny + j) * nz + kp]
                       + psi[(i * ny + j) * nz + km]
                       - 2 * psi[(i * ny + j) * nz + k]) / dz2;
            }
        }
    }
    for(;i < (nx - 1);i++)
    {
        im = i - 1;
        ip = i + 1;
        for(j = 0;j < 1;j++)
        {
            jm = ny - 1;
            jp = 1 % ny;
            for(k = 0;k < 1;k++)
            {
                km = nz - 1;
                kp = 1 % nz;
                L_psi[(i * ny + j) * nz + k] =
                      (psi[(ip * ny + j) * nz + k]
                       + psi[(im * ny + j) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dx2
                    + (psi[(i * ny + jp) * nz + k]
                       + psi[(i * ny + jm) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dy2
                    + (psi[(i * ny + j) * nz + kp]
                       + psi[(i * ny + j) * nz + km]
                       - 2 * psi[(i * ny + j) * nz + k]) / dz2;
            }
            for(;k < (nz - 1);k++)
            {
                km = k - 1;
                kp = k + 1;
                L_psi[(i * ny + j) * nz + k] =
                      (psi[(ip * ny + j) * nz + k]
                       + psi[(im * ny + j) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dx2
                    + (psi[(i * ny + jp) * nz + k]
                       + psi[(i * ny + jm) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dy2
                    + (psi[(i * ny + j) * nz + kp]
                       + psi[(i * ny + j) * nz + km]
                       - 2 * psi[(i * ny + j) * nz + k]) / dz2;
            }
            for(;k < nz;k++)
            {
                km = nz - 2;
                kp = 0;
                L_psi[(i * ny + j) * nz + k] =
                      (psi[(ip * ny + j) * nz + k]
                       + psi[(im * ny + j) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dx2
                    + (psi[(i * ny + jp) * nz + k]
                       + psi[(i * ny + jm) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dy2
                    + (psi[(i * ny + j) * nz + kp]
                       + psi[(i * ny + j) * nz + km]
                       - 2 * psi[(i * ny + j) * nz + k]) / dz2;
            }
        }
        for(;j < (ny - 1);j++)
        {
            jm = j - 1;
            jp = j + 1;
            for(k = 0;k < 1;k++)
            {
                km = nz - 1;
                kp = 1 % nz;
                L_psi[(i * ny + j) * nz + k] =
                      (psi[(ip * ny + j) * nz + k]
                       + psi[(im * ny + j) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dx2
                    + (psi[(i * ny + jp) * nz + k]
                       + psi[(i * ny + jm) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dy2
                    + (psi[(i * ny + j) * nz + kp]
                       + psi[(i * ny + j) * nz + km]
                       - 2 * psi[(i * ny + j) * nz + k]) / dz2;
            }
            for(;k < (nz - 1);k++)
            {
                km = k - 1;
                kp = k + 1;
                L_psi[(i * ny + j) * nz + k] =
                      (psi[(ip * ny + j) * nz + k]
                       + psi[(im * ny + j) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dx2
                    + (psi[(i * ny + jp) * nz + k]
                       + psi[(i * ny + jm) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dy2
                    + (psi[(i * ny + j) * nz + kp]
                       + psi[(i * ny + j) * nz + km]
                       - 2 * psi[(i * ny + j) * nz + k]) / dz2;
            }
            for(;k < nz;k++)
            {
                km = nz - 2;
                kp = 0;
                L_psi[(i * ny + j) * nz + k] =
                      (psi[(ip * ny + j) * nz + k]
                       + psi[(im * ny + j) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dx2
                    + (psi[(i * ny + jp) * nz + k]
                       + psi[(i * ny + jm) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dy2
                    + (psi[(i * ny + j) * nz + kp]
                       + psi[(i * ny + j) * nz + km]
                       - 2 * psi[(i * ny + j) * nz + k]) / dz2;
            }
        }
        for(;j < ny;j++)
        {
            jm = ny - 2;
            jp = 0;
            for(k = 0;k < 1;k++)
            {
                km = nz - 1;
                kp = 1 % nz;
                L_psi[(i * ny + j) * nz + k] =
                      (psi[(ip * ny + j) * nz + k]
                       + psi[(im * ny + j) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dx2
                    + (psi[(i * ny + jp) * nz + k]
                       + psi[(i * ny + jm) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dy2
                    + (psi[(i * ny + j) * nz + kp]
                       + psi[(i * ny + j) * nz + km]
                       - 2 * psi[(i * ny + j) * nz + k]) / dz2;
            }
            for(;k < (nz - 1);k++)
            {
                km = k - 1;
                kp = k + 1;
                L_psi[(i * ny + j) * nz + k] =
                      (psi[(ip * ny + j) * nz + k]
                       + psi[(im * ny + j) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dx2
                    + (psi[(i * ny + jp) * nz + k]
                       + psi[(i * ny + jm) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dy2
                    + (psi[(i * ny + j) * nz + kp]
                       + psi[(i * ny + j) * nz + km]
                       - 2 * psi[(i * ny + j) * nz + k]) / dz2;
            }
            for(;k < nz;k++)
            {
                km = nz - 2;
                kp = 0;
                L_psi[(i * ny + j) * nz + k] =
                      (psi[(ip * ny + j) * nz + k]
                       + psi[(im * ny + j) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dx2
                    + (psi[(i * ny + jp) * nz + k]
                       + psi[(i * ny + jm) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dy2
                    + (psi[(i * ny + j) * nz + kp]
                       + psi[(i * ny + j) * nz + km]
                       - 2 * psi[(i * ny + j) * nz + k]) / dz2;
            }
        }
    }
    for(;i < nx;i++)
    {
        im = nx - 2;
        ip = 0;
        for(j = 0;j < 1;j++)
        {
            jm = ny - 1;
            jp = 1 % ny;
            for(k = 0;k < 1;k++)
            {
                km = nz - 1;
                kp = 1 % nz;
                L_psi[(i * ny + j) * nz + k] =
                      (psi[(ip * ny + j) * nz + k]
                       + psi[(im * ny + j) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dx2
                    + (psi[(i * ny + jp) * nz + k]
                       + psi[(i * ny + jm) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dy2
                    + (psi[(i * ny + j) * nz + kp]
                       + psi[(i * ny + j) * nz + km]
                       - 2 * psi[(i * ny + j) * nz + k]) / dz2;
            }
            for(;k < (nz - 1);k++)
            {
                km = k - 1;
                kp = k + 1;
                L_psi[(i * ny + j) * nz + k] =
                      (psi[(ip * ny + j) * nz + k]
                       + psi[(im * ny + j) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dx2
                    + (psi[(i * ny + jp) * nz + k]
                       + psi[(i * ny + jm) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dy2
                    + (psi[(i * ny + j) * nz + kp]
                       + psi[(i * ny + j) * nz + km]
                       - 2 * psi[(i * ny + j) * nz + k]) / dz2;
            }
            for(;k < nz;k++)
            {
                km = nz - 2;
                kp = 0;
                L_psi[(i * ny + j) * nz + k] =
                      (psi[(ip * ny + j) * nz + k]
                       + psi[(im * ny + j) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dx2
                    + (psi[(i * ny + jp) * nz + k]
                       + psi[(i * ny + jm) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dy2
                    + (psi[(i * ny + j) * nz + kp]
                       + psi[(i * ny + j) * nz + km]
                       - 2 * psi[(i * ny + j) * nz + k]) / dz2;
            }
        }
        for(;j < (ny - 1);j++)
        {
            jm = j - 1;
            jp = j + 1;
            for(k = 0;k < 1;k++)
            {
                km = nz - 1;
                kp = 1 % nz;
                L_psi[(i * ny + j) * nz + k] =
                      (psi[(ip * ny + j) * nz + k]
                       + psi[(im * ny + j) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dx2
                    + (psi[(i * ny + jp) * nz + k]
                       + psi[(i * ny + jm) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dy2
                    + (psi[(i * ny + j) * nz + kp]
                       + psi[(i * ny + j) * nz + km]
                       - 2 * psi[(i * ny + j) * nz + k]) / dz2;
            }
            for(;k < (nz - 1);k++)
            {
                km = k - 1;
                kp = k + 1;
                L_psi[(i * ny + j) * nz + k] =
                      (psi[(ip * ny + j) * nz + k]
                       + psi[(im * ny + j) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dx2
                    + (psi[(i * ny + jp) * nz + k]
                       + psi[(i * ny + jm) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dy2
                    + (psi[(i * ny + j) * nz + kp]
                       + psi[(i * ny + j) * nz + km]
                       - 2 * psi[(i * ny + j) * nz + k]) / dz2;
            }
            for(;k < nz;k++)
            {
                km = nz - 2;
                kp = 0;
                L_psi[(i * ny + j) * nz + k] =
                      (psi[(ip * ny + j) * nz + k]
                       + psi[(im * ny + j) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dx2
                    + (psi[(i * ny + jp) * nz + k]
                       + psi[(i * ny + jm) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dy2
                    + (psi[(i * ny + j) * nz + kp]
                       + psi[(i * ny + j) * nz + km]
                       - 2 * psi[(i * ny + j) * nz + k]) / dz2;
            }
        }
        for(;j < ny;j++)
        {
            jm = ny - 2;
            jp = 0;
            for(k = 0;k < 1;k++)
            {
                km = nz - 1;
                kp = 1 % nz;
                L_psi[(i * ny + j) * nz + k] =
                      (psi[(ip * ny + j) * nz + k]
                       + psi[(im * ny + j) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dx2
                    + (psi[(i * ny + jp) * nz + k]
                       + psi[(i * ny + jm) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dy2
                    + (psi[(i * ny + j) * nz + kp]
                       + psi[(i * ny + j) * nz + km]
                       - 2 * psi[(i * ny + j) * nz + k]) / dz2;
            }
            for(;k < (nz - 1);k++)
            {
                km = k - 1;
                kp = k + 1;
                L_psi[(i * ny + j) * nz + k] =
                      (psi[(ip * ny + j) * nz + k]
                       + psi[(im * ny + j) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dx2
                    + (psi[(i * ny + jp) * nz + k]
                       + psi[(i * ny + jm) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dy2
                    + (psi[(i * ny + j) * nz + kp]
                       + psi[(i * ny + j) * nz + km]
                       - 2 * psi[(i * ny + j) * nz + k]) / dz2;
            }
            for(;k < nz;k++)
            {
                km = nz - 2;
                kp = 0;
                L_psi[(i * ny + j) * nz + k] =
                      (psi[(ip * ny + j) * nz + k]
                       + psi[(im * ny + j) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dx2
                    + (psi[(i * ny + jp) * nz + k]
                       + psi[(i * ny + jm) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dy2
                    + (psi[(i * ny + j) * nz + kp]
                       + psi[(i * ny + j) * nz + km]
                       - 2 * psi[(i * ny + j) * nz + k]) / dz2;
            }
        }
    }
}

/*---------------------------------------------------------------------------*/
