/*---------------------------------------------------------------------------*/

/*!  \file  laplacian.h
     \brief  Laplacian operator for field.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_LAPLACIAN_H
#define INCLUDE_LAPLACIAN_H

#include "geometry.h"

/*---------------------------------------------------------------------------*/
/*!  \brief  operate Laplacian to a 1D field.
     \param  geometry  geometry for the field
     \param  psi  field
     \param  Li_psi Laplacian of \a psi  */
void operate_L_1d(field_geometry *geometry,double *psi,double *L_psi);
/*!  \brief  operate Laplacian to a 2D field.
     \param  geometry  geometry for the field
     \param  psi  field
     \param  Li_psi Laplacian of \a psi  */
void operate_L_2d(field_geometry *geometry,double *psi,double *L_psi);
/*!  \brief  operate Laplacian to a 3D field.
     \param  geometry  geometry for the field
     \param  psi  field
     \param  Li_psi Laplacian of \a psi  */
void operate_L_3d(field_geometry *geometry,double *psi,double *L_psi);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
