/*---------------------------------------------------------------------------*/

/*!  \file  matrix.c
     \brief  utility library for handling matrix.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005,2006 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include "xmalloc.h"
#include "matrix.h"

/*---------------------------------------------------------------------------*/
double **allocate_matrix(int n)
{
    int i;
    double **matrix;

    matrix = xmalloc(sizeof(double *) * n);
    matrix[0] = xcalloc(n * n, sizeof(double));
    for(i = 1;i < n;i++)
    {
        matrix[i] = matrix[0] + i * n;
    }

    return matrix;
}

/*---------------------------------------------------------------------------*/
double *allocate_vector(int n)
{
    return xcalloc(n, sizeof(double));
}

/*---------------------------------------------------------------------------*/
void free_matrix(double **matrix)
{
    free(matrix[0]);
    free(matrix);
}

/*---------------------------------------------------------------------------*/
void free_vector(double *vector)
{
    free(vector);
}

/*---------------------------------------------------------------------------*/
