/*---------------------------------------------------------------------------*/

/*!  \file  mt.h
     \brief  the Mersenne Twister random number library.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005 Takashi Uneyama  */

/* Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

   1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

   3. The names of its contributors may not be used to endorse or promote 
   products derived from this software without specific prior written 
   permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
   OF THE POSSIBILITY OF SUCH DAMAGE.   */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_MT_H
#define INCLUDE_MT_H

/*---------------------------------------------------------------------------*/
/*!  \brief  initialize random nubmer generator with a seed \a s.
     \param  s  seed for initialization  */
void mtinit(unsigned long s);
/*!  \brief  initialize the random number generator by an array \a init_key with array-length \a key_length.
     \param  init_key  array of seeds for initializing keys
     \param  key_length  length of \a init_key  */
void mtinit_by_array(unsigned long init_key[],unsigned long key_length);
/*!  \brief  generates a random number on [0,0xffffffff]-interval.  */
unsigned long mtrand_int32(void);
/*!  \brief  generates a random number on [0,0x7fffffff]-interval.  */
long mtrand_int31(void);
/*!  \brief  generates a random number on [0,1]-real-interval.  */
double mtrand_real1(void);
/*!  \brief  generates a random number on [0,1)-real-interval.  */
double mtrand_real2(void);
/*!  \brief  generates a random number on (0,1)-real-interval.  */
double mtrand_real3(void);
/*!  \brief  generates a random number on [0,1) with 53-bit resolution.  */
double mtrand_res53(void);

/*---------------------------------------------------------------------------*/

#endif
/*---------------------------------------------------------------------------*/
