/*---------------------------------------------------------------------------*/

/*!  \file  multigrid.h
     \brief  the multigrid infomation for the field.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_MULTIGRID_H
#define INCLUDE_MULTIGRID_H

#include "geometry.h"

/*---------------------------------------------------------------------------*/
/*!  \brief  geometry for a field  */
typedef struct multigrid_field
{
        /*!  \brief  dimension of a field.  */
    int dimension;
        /*!  \brief  multigrid level.  */
    int level;
        /*!  \brief  number of division.  */
    int *nx,*ny,*nz;
        /*!  \brief  length of a edge.  */
    double lx,ly,lz;
        /*!  \brief  approximate solution.  */
    double **u;
        /*!  \brief  approximate difference.  */
    double **v;
        /*!  \brief  source field.  */
    double **f;
        /*!  \brief  residual.  */
    double **r;
} multigrid_field;

/*---------------------------------------------------------------------------*/
/*!  \brief  allocate a new multigrid field.
     \param  geometry  geometry of a field  */
multigrid_field *allocate_multigrid_field(field_geometry *geometry);
/*!  \brief  free a field multigrid field.
     \param  multigrid  multigrid field to free  */
void free_multigrid_field(multigrid_field *multigrid);
/*!  \brief  returns a nubmer of lattice points for the grid of \a level.
     \param  multigrid  multigrid field
     \param  level  grid level  */
int lattice_point_number_multigrid(multigrid_field *multigrid,int level);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
