/*---------------------------------------------------------------------------*/

/*!  \file  nrand.c
     \brief  normal distribution random numbers library.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2006 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#ifdef HAVE_MATH_H
#include <math.h>
#endif

#include "nrand.h"
#include "mt.h"

/*---------------------------------------------------------------------------*/
double nrand(void)
{
    static int number_exists = FALSE;
    static double x,y,r;

        /*  Box-Muller method.  */
    if(number_exists)
    {
        number_exists = FALSE;
        return y * r;
    }
    else
    {
        do
        {
            x = 2.0 * mtrand_real2() - 1.0;
            y = 2.0 * mtrand_real2() - 1.0;
            r = x * x + y * y;
        }
        while((r > 1.0) || (r == 0.0));

        r = sqrt(-2.0 * log(r) / r);
        number_exists = TRUE;
        return x * r;
    }
}

/*---------------------------------------------------------------------------*/
