/*---------------------------------------------------------------------------*/

/*!  \file  polymer.h
     \brief  some definitions of structures for polymers.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005,2006 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_POLYMER_H
#define INCLUDE_POLYMER_H

/*---------------------------------------------------------------------------*/
/*!  \brief  information of a polymer chain.  */
typedef struct polymer_chain
{
        /*!  \brief  number of subchains.  */
    int ns;
        /*!  \brief  degree of polymerization.  */
    double N;
        /*!  \brief  block ratio.  */
    double *f;
        /*!  \brief  effective bond length (Kuhn length).  */
    double *b;
        /*!  \brief  mean square distance between subchains.  */
    double **l2;
        /*!  \brief  cut off length for the long range interaction.  */
    double lambda;
        /*!  \brief  monomer species in the subchain.  */
    int *monomer;
} polymer_chain;

/*---------------------------------------------------------------------------*/
/*!  \brief  allocate a new polymer chain.
     \param  n  number of subchains  */
polymer_chain *allocate_polymer_chain(int n);
/*!  \brief  free a polymer chain.
     \param  chain  a polymer chain to free  */
void free_polymer_chain(polymer_chain *chain);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
