/*---------------------------------------------------------------------------*/

/*!  \file  propagator.h
     \brief  calculate the free propagator for Gaussian chains.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_PROPAGATOR_H
#define INCLUDE_PROPAGATOR_H

#include "polymer.h"

/*---------------------------------------------------------------------------*/
/*!  \brief  propagator for polymer chain.  */
typedef struct chain_propagator
{
        /*!  \brief  number of subchains  */
    int n;
        /*!  \brief  coefficient of 1 / q^2   */
    double **A;
        /*!  \brief  independent of q */
    double **C;
        /*!  \brief  coefficient of q^2  */
    double **B;
} chain_propagator;

/*---------------------------------------------------------------------------*/
/*!  \brief  calculate a chain propagator for \a chain.
     \param  chain  polymer chain
     \param  propagator  propagator for polymer chain  */
void calculate_chain_propagator(polymer_chain *chain,
                                chain_propagator *propagator);
/*!  \brief  allocate a new chain propagator.
     \param  n  number of subchains in a polymer  */
chain_propagator *allocate_chain_propagator(int n);
/*!  \brief  free a chain propagator.
     \param  propagator  propagator to free  */
void free_chain_propagator(chain_propagator *propagator);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
