/*---------------------------------------------------------------------------*/

/*!  \file  simulator.h
     \brief  a statics simulator.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_SIMULATOR_H
#define INCLUDE_SIMULATOR_H

#include "chemicalpotential.h"
#include "condition.h"
#include "externalforce.h"
#include "iofiles.h"
#include "mgpoisson.h"

/*--------------------------------------------------------------------------*/
/*!  \brief  do a static simulation.
     \param  geometry  geometry of the field
     \param  condition  simulation condition
     \param  iofiles  simulation iofiles
     \param  blend  polymer blend
     \param  density  density field
     \param  force  external force field
     \param  plan  plan for FFTW
     \param  potential  chemical potential field
     \param  mgcondition  multigrid Poisson solver condition  */
int simulate(field_geometry *geometry,simulation_condition *condition,
             simulation_iofiles *iofiles,
             polymer_blend *blend,density_field *density,
             external_force *force,
             field_fftw_plan *plan,chemical_potential *potential,
             multigrid_condition *mgcondition);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
