/*---------------------------------------------------------------------------*/

/*!  \file  textread.c
     \brief  output the field data as ascii text format.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005,2006,2018 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#include <stdio.h>

#ifdef HAVE_ZLIB_H
#include <zlib.h>
#endif

#include "textread.h"
#include "zget.h"

/*---------------------------------------------------------------------------*/
void read_density_field_text(char *filename,density_field *density)
{
    gzFile fp;
    int p,i,r;
    int np,*ns;
    int n;
    double ***psi;
    double psi_min;

    np = density->blend->np;
    ns = density->blend->ns;
    psi = density->psi;
    n = lattice_point_number(density->geometry);


    fp = gzopen(filename,"rb");
    if(fp == NULL)
        return;

    for(r = 0;r < n;r++)
    {
        for(p = 0;p < np;p++)
        {
            for(i = 0;i < ns[p];i++)
            {
                gzgetdouble(fp,&psi[p][i][r]);
            }
        }
    }

    gzclose(fp);


    for(p = 0;p < np;p++)
    {
        for(i = 0;i < ns[p];i++)
        {
            psi_min = 0.0;
            for(r = 0;r < n;r++)
            {
                if(psi[p][i][r] < psi_min)
                    psi_min = psi[p][i][r];
            }

            if(psi_min < 0.0)
            {
                for(r = 0;r < n;r++)
                {
                    psi[p][i][r] -= psi_min;
                }
            }
        }
    }
}

/*---------------------------------------------------------------------------*/
void read_chemical_potential_text(char *filename,
                                   chemical_potential *potential)
{
    gzFile fp;
    int p,i,r;
    int np,*ns;
    int n;
    double ***mu;

    np = potential->blend->np;
    ns = potential->blend->ns;
    mu = potential->mu;
    n = lattice_point_number(potential->geometry);


    fp = gzopen(filename,"rb");

    for(r = 0;r < n;r++)
    {
        for(p = 0;p < np;p++)
        {
            for(i = 0;i < ns[p];i++)
            {
                gzgetdouble(fp,&mu[p][i][r]);
            }
        }
    }

    gzclose(fp);
}

/*---------------------------------------------------------------------------*/
void read_external_force_text(char *filename,
                              external_force *force)
{
    gzFile fp;
    int p,i,r;
    int np,*ns;
    int n;
    double ***F;

    np = force->blend->np;
    ns = force->blend->ns;
    F = force->F;
    n = lattice_point_number(force->geometry);


    fp = gzopen(filename,"rb");
    if(fp == NULL)
    {
        for(r = 0;r < n;r++)
        {
            for(p = 0;p < np;p++)
            {
                for(i = 0;i < ns[p];i++)
                {
                    F[p][i][r] = 0;
                }
            }
        }
        return;
    }

    for(r = 0;r < n;r++)
    {
        for(p = 0;p < np;p++)
        {
            for(i = 0;i < ns[p];i++)
            {
                gzgetdouble(fp,&F[p][i][r]);
            }
        }
    }

    gzclose(fp);
}

/*---------------------------------------------------------------------------*/
