/*---------------------------------------------------------------------------*/

/*!  \file  textwrite.c
     \brief  output the field data as ascii text format.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005-2007 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#include <stdio.h>
#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifdef HAVE_ZLIB_H
#include <zlib.h>
#endif

#include "textwrite.h"

/*---------------------------------------------------------------------------*/
void write_psi_field_text(char *filename,density_field *density)
{
    gzFile fp;
    int p,i,x;
    int np,*ns;
    int n;
    double ***psi;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    np = density->blend->np;
    ns = density->blend->ns;
    psi = density->psi;
    n = lattice_point_number(density->geometry);


    fp = gzopen(filename,"wb");

    for(x = 0;x < n;x++)
    {
        for(p = 0;p < np;p++)
        {
            for(i = 0;i < ns[p];i++)
            {
                gzprintf(fp,"%g ",psi[p][i][x]);
            }
        }
        gzprintf(fp,"\n");
    }

    gzclose(fp);
}

/*---------------------------------------------------------------------------*/
void write_chemical_potential_text(char *filename,
                                   chemical_potential *potential)
{
    gzFile fp;
    int p,i,x;
    int np,*ns;
    int n;
    double ***mu;

    np = potential->blend->np;
    ns = potential->blend->ns;
    mu = potential->mu;
    n = lattice_point_number(potential->geometry);


    fp = gzopen(filename,"wb");

    for(x = 0;x < n;x++)
    {
        for(p = 0;p < np;p++)
        {
            for(i = 0;i < ns[p];i++)
            {
                gzprintf(fp,"%g ",mu[p][i][x]);
            }
        }
        gzprintf(fp,"\n");
    }

    gzclose(fp);
}

/*---------------------------------------------------------------------------*/
void write_free_energy_text(char *filename,double fe,double fe_longrange,
                            double fe_local,double fe_gradient)
{
    FILE *fp;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    fp = fopen(filename,"w");

    fprintf(fp,"%g %g %g %g\n",fe,fe_longrange,fe_local,fe_gradient);

    fclose(fp);
}

/*---------------------------------------------------------------------------*/
void write_geometry_text(char *filename,field_geometry *geometry)
{
    FILE *fp;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    fp = fopen(filename,"w");

    fprintf(fp,"%d\n",geometry->dimension);
    switch(geometry->dimension)
    {
        case 1:
            fprintf(fp,"%d\n",geometry->nx);
            fprintf(fp,"%g\n",geometry->lx);
            break;

        case 2:
            fprintf(fp,"%d %d\n",geometry->nx,geometry->ny);
            fprintf(fp,"%g %g\n",geometry->lx,geometry->ly);
            break;

        case 3:
            fprintf(fp,"%d %d %d\n",geometry->nx,geometry->ny,geometry->nz);
            fprintf(fp,"%g %g %g\n",geometry->lx,geometry->ly,geometry->lz);
            break;

        default:
            break;
    }

    fclose(fp);
}

/*---------------------------------------------------------------------------*/
