/*---------------------------------------------------------------------------*/

/*!  \file  textwrite.h
     \brief  output the field data as ascii text format.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_TEXTWRITE_H
#define INCLUDE_TEXTWRITE_H

#include "chemicalpotential.h"

/*---------------------------------------------------------------------------*/
/*!  \brief  write data of \a density to \a filename.
     \param  filename  file name
     \param  density  density field  */
void write_psi_field_text(char *filename,density_field *density);
/*!  \brief  write data of \a potential to \a filename.
     \param  filename  file name
     \param  potential  chemical potential field  */
void write_chemical_potential_text(char *filename,
                                   chemical_potential *potential);
/*!  \brief  write the free energy \fe to \a filename.
     \param  filename  file name
     \param  fe  free energy  */
void write_free_energy_text(char *filename,double fe,double fe_longrange,
                            double fe_local,double fe_gradient);
/*!  \brief  write the geometry to \a filename.
     \param  filename  file name
     \param  geometry  field geometry  */
void write_geometry_text(char *filename,field_geometry *geometry);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
