/*---------------------------------------------------------------------------*/

/*!  \file  field.c
     \brief  the polymer density / chemical potential field information.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_MATH_H
#include <math.h>
#endif

#include "xmalloc.h"
#include "field.h"

/*---------------------------------------------------------------------------*/
double **allocate_field_double(field_geometry *geometry,int n)
{
    double **u;
    int i;

    u = xmalloc(sizeof(double *) * n);
    u[0] = xmalloc(sizeof(double) * n * lattice_point_number(geometry));

    for(i = 1;i < n;i++)
    {
        u[i] = u[i - 1] + lattice_point_number(geometry);
    }

    return u;
}

/*---------------------------------------------------------------------------*/
void free_field_double(double **u)
{
    free(u[0]);
    free(u);
}

/*---------------------------------------------------------------------------*/
float **allocate_field_float(field_geometry *geometry,int n)
{
    float **u;
    int i;

    u = xmalloc(sizeof(float *) * n);
    u[0] = xmalloc(sizeof(float) * n * lattice_point_number(geometry));

    for(i = 1;i < n;i++)
    {
        u[i] = u[i - 1] + lattice_point_number(geometry);
    }

    return u;
}

/*---------------------------------------------------------------------------*/
void free_field_float(float **u)
{
    free(u[0]);
    free(u);
}

/*---------------------------------------------------------------------------*/
void convert_field_double_to_float(double **u_double,float **u_float,
                                   field_geometry *geometry,int n)
{
    int i;

    for(i = 0;i < lattice_point_number(geometry) * n;i++)
    {
        (*u_float)[i] = (*u_double)[i];
    }
}

/*---------------------------------------------------------------------------*/
void convert_field_float_to_double(float **u_float,double **u_double,
                                   field_geometry *geometry,int n)
{
    int i;

    for(i = 0;i < lattice_point_number(geometry) * n;i++)
    {
        (*u_double)[i] = (*u_float)[i];
    }
}

/*---------------------------------------------------------------------------*/
void convert_field_sqrt_float(float **u,field_geometry *geometry,int n)
{
    int i;

    for(i = 0;i < lattice_point_number(geometry) * n;i++)
    {
        (*u)[i] = ((*u)[i] > 0.0) ? sqrt((*u)[i]) : 0.0;
    }
}

/*---------------------------------------------------------------------------*/
void convert_field_sqrt_double(double **u,field_geometry *geometry,int n)
{
    int i;

    for(i = 0;i < lattice_point_number(geometry) * n;i++)
    {
        (*u)[i] = ((*u)[i] > 0.0) ? sqrt((*u)[i]) : 0.0;
    }
}

/*---------------------------------------------------------------------------*/
