/*---------------------------------------------------------------------------*/

/*!  \file  geometry.c
     \brief  the geometry infomation for the field.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include "xmalloc.h"
#include "geometry.h"

/*---------------------------------------------------------------------------*/
field_geometry *allocate_field_geometry(void)
{
    field_geometry *geometry;

    geometry = xmalloc(sizeof(field_geometry));

    return geometry;
}

/*---------------------------------------------------------------------------*/
void free_field_geometry(field_geometry *geometry)
{
    if(geometry == NULL)
        return;

    free(geometry);
}

/*---------------------------------------------------------------------------*/
int lattice_point_number(field_geometry *geometry)
{
    int n;

    switch(geometry->dimension)
    {
        case 1:
            n = geometry->n[0];
            break;

        case 2:
            n = geometry->n[0] * geometry->n[1];
            break;

        case 3:
            n = geometry->n[0] * geometry->n[1] * geometry->n[2];
            break;

        default:
            n = 0;
            break;
    }

    return n;
}

/*---------------------------------------------------------------------------*/
